#include <iostream>
#include <fstream>
#include <cstdlib>
#include <ctime>
#include <unistd.h>
#include "csp-scanner.hpp"
#include "csp-parser.hpp"
#include "csp-yytname.hpp"
#include "csp-symtable.hpp"

using namespace std;
using namespace CSP;

int main(int argc, char** argv) {
   char* cmdname = *argv++; --argc;
   if (argc != 1) {
      cerr << "Usage: " << cmdname << " filename" << endl;
      exit(1);
   }

   char* fname = *argv++; --argc;
   ifstream fin(fname);
   if (!fin) {
      cerr << cmdname << ": unable to open " << fname <<
	 " for reading" << endl;
      exit(1);
   }
   SymTable symtab;
   std::string filename(fname);
   Scanner scanner(fin, filename, symtab);

   /* initialize pseudo-randomgenerator */
   std::srand(time(0) ^ getpid());

   ProcessPtr process;
   parser p(scanner, symtab, process);
   if (p.parse() == 0) {
      cout << "Tracing: " << process << endl;
      cout << "Alphabet: " << process->get_alphabet() << endl;
      cout << "Acceptable: " << process->acceptable() << endl;
      if (!process->accepts_success()) {
	 std::string event;
	 while (cin >> event) {
	    if (process->get_alphabet().is_member(event)) {
	       process = process->proceed(event);
	       if (!process) {
		  cerr << "cannot accept " << event << endl;
		  exit(1);
	       }
	       if (process->accepts_success()) break;
	       cout << "Process: " << process << endl;
	       cout << "Acceptable: " << process->acceptable() << endl;
	    } else {
	       cout << "Not in alphabet: " << event << endl;
	    }
	 }
      }
      cout << "OK" << endl;
   }
}
